#!/usr/bin/bash

function stop_slapd() {
  systemctl status slapd > /dev/null 2>&1
  if [ $? -eq 0 ]
  then
    echo 'INFO: slapd service is running and needs to be stopped.'
    echo 'INFO: Stopping slapd service...'
    
    systemctl stop slapd
    if [ $? -ne 0 ]
    then
      echo 'ERR_: Failed to stop slapd service.'
      exit 4
    fi
    echo 'INFO: Stopped slapd service.'
  fi
}

function start_slapd() {
  systemctl start slapd
  if [ $? -ne 0 ]
  then
    echo 'ERR_: Failed to start slapd service.'
    exit 5
  fi
  
  echo 'INFO: Started slapd service.'
}

function confirm_password() {
  while true; do
    read -s -p "Password: " password
    echo
    read -s -p "Confirm password: " password2
    echo
    [ "$password" = "$password2" ] && break
    echo 'WARN: Passwords do not match.'
  done
}

function clear_slapd() {
  echo 'INFO: Clearing previous openldap-servers/slapd install.'

  systemctl disable slapd
  if [ $? -ne 0 ]
  then
    echo 'ERR_: Failed to disable slapd service.'
    exit 7
  fi
  rm -rf /etc/openldap/slapd.d
  rm -f /var/lib/ldap/*

  echo 'INFO: Removing openldap-servers.'
  yum -y remove openldap-servers > /dev/null 2>&1
  if [ $? -ne 0 ]
  then
    echo 'WARN: Failed to remove openldap-servers.'
  fi

  echo 'INFO: Re-installing openldap-servers.'
  yum -y install openldap-servers > /dev/null 2>&1
}

# Configuring OpenLDAP script

echo 'INFO: OpenLDAP configuration script.'

currentdir=$(dirname -- "$( readlink -f -- "$0"; )";)

stop_slapd

clear_slapd

echo 'INFO: Modify /etc/openldap/slapd.d/cn=config/olcDatabase={0}config.ldif file...'

# cp /home/ec2-user/Documents/olcDatabase={0}config.ldif /etc/openldap/slapd.d/cn=config

sed -i '/CRC.*/d' /etc/openldap/slapd.d/cn=config/olcDatabase={0}config.ldif
sed -i '/olcAccess:.*/c\olcAccess: {0}to * by dn.exact=gidNumber=0+uidNumber=0,cn=peercred,cn=external,cn=auth manage by * break' /etc/openldap/slapd.d/cn=config/olcDatabase={0}config.ldif
sed -i '/ ,cn=auth.*/d' /etc/openldap/slapd.d/cn=config/olcDatabase={0}config.ldif

echo 'INFO: Starting slapd service.'

start_slapd

echo 'INFO: Started slapd service.'

echo 'INFO: Please create a password for your LDAP directory configuration.'

#confirm_password
password="configpw"

echo 'INFO: Collecting slappasswd secret.'

secret=$(slappasswd -s $password)

echo "INFO: slappasswd secret is $secret"

echo 'INFO: Modify chrootpwd.ldif with secret'

sed -i "/olcRootPW:.*/c\olcRootPW: $secret" $currentdir/chrootpwd.ldif

echo 'INFO: Execute ldapadd commands'

ldapadd -Y EXTERNAL -H ldapi:/// -f $currentdir/chrootpwd.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/core.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/corba.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/cosine.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/duaconf.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/dyngroup.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/inetorgperson.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/java.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/misc.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/nis.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/openldap.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f /etc/openldap/schema/collective.ldif > /dev/null 2>&1
ldapadd -Y EXTERNAL -H ldapi:/// -f $currentdir/ppolicy.ldif > /dev/null 2>&1

echo 'INFO: Please create a password for administrator connection.'

#confirm_password
password="password"

echo 'INFO: Collecting slappasswd secret.'

secret=$(slappasswd -s $password)

echo "INFO: slappasswd secret is $secret"

echo 'INFO: Modify backend.ldif with secret.'
sed -i "/olcRootPW:.*/c\olcRootPW: $secret" $currentdir/backend.ldif

echo 'INFO: ldapadd backend.ldif.'
ldapadd -Y EXTERNAL -H ldapi:/// -f $currentdir/backend.ldif > /dev/null 2>&1

stop_slapd

echo 'INFO: Creating schema folder.'
cp -r /etc/openldap/schema $currentdir

echo 'INFO: Create the mfds.schema using MFDS.'
mfds -l "dc=secldap,dc=com" 2 $currentdir/schema/mfds.schema > /dev/null 2>&1
if [ $? -ne 0 ]
then
  echo 'ERR_: Failed to run mfds -l command.'
  exit 6
fi

echo 'INFO: Remove cn=config directory.'
# adaptation n°1 
# rm -rf $currentdir/schema/cn=config

rm -rf $currentdir/schema/slaptest
mkdir $currentdir/schema/slaptest

cd $currentdir/schema

echo 'INFO: Convert the schema with slaptest.'
slaptest -f $currentdir/schema/schema_convert.conf -F $currentdir/schema/slaptest
cd $currentdir

echo 'INFO: copy cn={12}container.ldif and cn={13}mfds.schema.ldif to /etc/openldap/slapd.d/cn=config/cn=schema.'
cp $currentdir/schema/slaptest/cn=config/cn=schema/cn={12}container.ldif /etc/openldap/slapd.d/cn=config/cn=schema
cp $currentdir/schema/slaptest/cn=config/cn=schema/cn={13}mfds.ldif /etc/openldap/slapd.d/cn=config/cn=schema

echo 'INFO: Change owner of /etc/openldap/slapd.d'
chown -R ldap /etc/openldap/slapd.d

echo 'INFO: Change permissions of /etc/openldap/slapd.d'
chmod -R 700 /etc/openldap/slapd.d

start_slapd

echo 'INFO: Enable slapd service.'
systemctl enable slapd

echo 'INFO: Make log folder'
rm -rf $currentdir/schema/log
mkdir $currentdir/schema/log

echo 'INFO: ldapadd ./schema/log/top.ldif. Log written to ./schema/log/top.log.'
ldapadd -v -D "cn=Manager,dc=secldap,dc=com" -w $password -f $currentdir/schema/top.ldif -h localhost:389 > $currentdir/schema/log/top.log

echo 'INFO: ldapadd mf-containers.ldif. Log written to ./schema/log/mf-containers.log.'
ldapadd -v -D "cn=Manager,dc=secldap,dc=com" -w $password -f $currentdir/schema/mf-containers.ldif -h localhost:389 > $currentdir/schema/log/containers.log

echo 'INFO: Export mfds-users.ldif from MFDS.'
rm -f $currentdir/schema/mfds-users.ldif
mfds -e "cn=Micro Focus,dc=secldap,dc=com" "cn=Enterprise Server Users" "cn=Enterprise Server User Groups" "cn=Enterprise Server Resources" 2 "$currentdir/schema/mfds-users.ldif" > /dev/null 2>&1

echo 'INFO: ldapadd mfds-users.ldif. Log written to ./schema/log/mfds-users.log.'
ldapadd -v -D "cn=Manager,dc=secldap,dc=com" -w $password -f $currentdir/schema/mfds-users.ldif -h localhost:389 -c > $currentdir/schema/log/mfds-users.log

echo 'INFO: Modify /opt/ed92/etc/es_default_ldap_openldap.ldf.'
rm $currentdir/schema/es_default_ldap_openldap.ldif
sed 's/DC=X/CN=Micro Focus,dc=secldap,dc=com/' /opt/ed92/etc/es_default_ldap_openldap.ldf > $currentdir/schema/es_default_ldap_openldap.ldif

echo 'INFO: ldapadd es_default_ldap_openldap.ldf. Log written to log/es_default_ldap_openldap.log.'
ldapadd -v -D "cn=Manager,dc=secldap,dc=com" -w $password -f $currentdir/schema/es_default_ldap_openldap.ldif -h localhost:389 -c > $currentdir/schema/log/es_default_ldap_openldap.log

echo 'INFO: Output written to ./schema/es_default_ldap_openldap.txt.'
